import 'dart:collection';

import 'package:meta/meta.dart';

import 'module.dart';
import 'providers.dart';
import '../error/errors.dart';

abstract class MiodiComponent {
  final Map<Type, Provider> factoryConfiguration = HashMap();
  final Map<Type, Provider> scopeConfiguration = HashMap();
  final Map<Type, Provider> singletonConfiguration = HashMap();
  final Map<Type, Provider> customConfiguration = HashMap();

  @mustCallSuper
  MiodiComponent() {
    modules.forEach((module) {
      factoryConfiguration.addAll(module.factoryConfiguration);
    });
    modules.forEach((module) {
      scopeConfiguration.addAll(module.scopeConfiguration);
    });
    modules.forEach((module) {
      singletonConfiguration.addAll(module.singletonConfig);
    });
    modules.forEach((module) {
      customConfiguration.addAll(module.customConfiguration);
    });
  }

  List<MiodiModule> get modules;

  T get<T>() {
    if (singletonConfiguration.containsKey(T))
      return singletonConfiguration[T].provide() as T;
    if (scopeConfiguration.containsKey(T))
      return scopeConfiguration[T].provide() as T;
    if (factoryConfiguration.containsKey(T))
      return factoryConfiguration[T].provide() as T;
    if (customConfiguration.containsKey(T))
      return customConfiguration[T].provide() as T;
    throw NotFoundProviderException(T);
  }
}

class DefaultMiodiComponent extends MiodiComponent {
  final List<MiodiModule> _modules;
  DefaultMiodiComponent(this._modules): super();

  @override
  List<MiodiModule> get modules => _modules;

}