import 'dart:async';
import 'dart:collection';

import '../error/errors.dart';
import 'component.dart';
import 'providers.dart';

class MiodiConfiguration {
  final Map<Type, Provider> _factoryConfiguration = HashMap();
  final Map<Type, Provider> _scopeConfiguration = HashMap();
  final Map<Type, Provider> _singletonConfiguration = HashMap();
  final Map<Type, Provider> _customConfiguration = HashMap();

  MiodiConfiguration(List<MiodiComponent> components) {
    components.forEach((component) {
      _factoryConfiguration.addAll(component.factoryConfiguration);
      _scopeConfiguration.addAll(component.scopeConfiguration);
      _singletonConfiguration.addAll(component.singletonConfiguration);
      _customConfiguration.addAll(component.customConfiguration);
    });
  }

  T provide<T>() {
    if (_singletonConfiguration.containsKey(T)){
      return _singletonConfiguration[T].provide();
    }
    if (_scopeConfiguration.containsKey(T)){
      return _scopeConfiguration[T].provide();
    }
    if (_factoryConfiguration.containsKey(T)){
      return _factoryConfiguration[T].provide();
    }
    if (_customConfiguration.containsKey(T)){
      return _customConfiguration[T].provide();
    }
    throw NotFoundProviderException(T);
  }

  Future<T> provideAsync<T>() async {
    return provide<T>();
  }
}