import 'dart:collection';

import 'providers.dart';

abstract class MiodiModule {
  final Map<Type, Provider> factoryConfiguration = HashMap();
  final Map<Type, Provider> singletonConfig = HashMap();
  final Map<Type, Provider> scopeConfiguration = HashMap();
  final Map<Type, Provider> customConfiguration = HashMap();

  MiodiModule() {
    providers.forEach((provider) {
      if (provider is SingletonProvider) {
        if (singletonConfig.containsKey(provider.provideType))
          singletonConfig[provider.provideType] = provider;
        else
          singletonConfig.addAll({provider.provideType: provider});
      } else if (provider is ScopeProvider) {
        if (scopeConfiguration.containsKey(provider.provideType)) {
          scopeConfiguration[provider.provideType] = provider;
        } else
          scopeConfiguration.addAll({provider.provideType: provider});
      } else if (provider is FactoryProvider) {
        if (factoryConfiguration.containsKey(provider.provideType)) {
          factoryConfiguration[provider.provideType] = provider;
        } else
          factoryConfiguration.addAll({provider.provideType: provider});
      } else {
        if (customConfiguration.containsKey(provider.provideType)) {
          customConfiguration[provider.provideType] = provider;
        } else
          customConfiguration.addAll({provider.provideType: provider});
      }
    });
  }

  List<Provider> get providers;
}

class DefaultMiodiModule extends MiodiModule {
  final List<Provider> _providers;
  DefaultMiodiModule(this._providers): super();

  @override
  List<Provider> get providers => _providers;

}