class FactoryProvider<T> extends Provider<T> {
  FactoryProvider(Type T, Creator<T> creator): super(T, creator);

  @override
  T provide() => creator();
}

class SingletonProvider<T> extends Provider<T> {
  T _instance;
  SingletonProvider(Type T, Creator<T> creator) : super(T, creator);

  @override
  T provide() {
    if (_instance == null) _instance = creator();
    return _instance;
  }
}

class ScopeProvider<T> extends Provider<T> {
  final Expando<T> _expando = Expando<T>();
  ScopeProvider(Type T, Creator<T> creator): super(T, creator);

  @override
  T provide() {
    if (_expando[this] == null) _expando[this] = creator();
    return _expando[this];
  }
}

abstract class Provider<T> {
  final Type _type;
  final Creator<T> _creator;
  Provider(this._type, this._creator);
  Type get provideType => _type;
  Creator<T> get creator => _creator;

  T provide();
}

typedef T Creator<T>();