library miodi;

import 'dart:async';
import 'core/component.dart';
import 'core/configuration.dart';
import 'core/module.dart';
import 'core/providers.dart';
import 'error/errors.dart';

export 'core/component.dart';
export 'core/module.dart';
export 'core/providers.dart';

Provider<T> singleton<T>(Type T, Creator<T> creator) =>
    SingletonProvider(T, creator);

Provider<T> factory<T>(Type T, Creator<T> creator) => FactoryProvider(T, creator);

Provider<T> scope<T>(Type T, Creator<T> creator) => ScopeProvider(T, creator);

MiodiModule module(List<Provider> providers) => DefaultMiodiModule(providers);

MiodiComponent component(List<MiodiModule> modules) => DefaultMiodiComponent(modules);


class Miodi {
  final MiodiConfiguration _configuration;
  static Miodi _instance;

  Miodi._(this._configuration);

  static void init(List<MiodiComponent> components) {
    if (_instance != null) return;
    _instance = Miodi._(MiodiConfiguration(components));
  }

  static T inject<T>() {
    if (_instance == null) throw NotInitializeMiodiException();
    return _instance._configuration.provide();
  }


  static Future<T> injectAsync<T>() async {
    return inject<T>();
  }
}
